@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2014, Luis Pedro Coelho <luis@luispedro.org>
+# Copyright (C) 2008-2019, Luis Pedro Coelho <luis@luispedro.org>
 # vim: set ts=4 sts=4 sw=4 expandtab smartindent:
 #
 # License: MIT
@@ -307,7 +307,7 @@ def cwatershed(surface, markers, Bc=None, return_lines=False):
     _verify_is_integer_type(markers, 'cwatershed')
     if surface.shape != markers.shape:
         raise ValueError('morph.cwatershed: Markers array should have the same shape as value array.')
-    markers = markers.astype(np.int64)
+    markers = np.asanyarray(markers, np.int64)
     Bc = get_structuring_elem(surface, Bc)
     return _morph.cwatershed(surface, markers, Bc, bool(return_lines))
 
@@ -319,7 +319,7 @@ def hitmiss(input, Bc, out=None, output=None):
 
     For a given pixel position, the hit&miss is ``True`` if, when ``Bc`` is
     overlaid on ``input``, centered at that position, the ``1`` values line up
-    with ``1``\ s, while the ``0``\ s line up with ``0``\ s (``2``\ s correspond to
+    with ``1``\s, while the ``0``\s line up with ``0``\s (``2``\s correspond to
     *don't care*).
 
     Examples
@@ -371,9 +371,12 @@ def hitmiss(input, Bc, out=None, output=None):
         else:
             Bc = Bc.astype(input.dtype)
 
-    if out is None and output is not None:
+    if out is None and output is not None: # pragma: no cover
         out = output
 
+    # We cannot call internal._get_output here because the conditions around
+    # dtypes are different from those implemented in `internal._get_output`
+
     if out is None:
         out = np.empty_like(input)
     else:
@@ -495,7 +498,7 @@ def close_holes(ref, Bc=None):
 
 def majority_filter(img, N=3, out=None, output=None):
     '''
-    filtered = majority_filter(img, N=3, out={np.empty(img.shape, np.bool)})
+    filtered = majority_filter(img, N=3, out={np.empty(img.shape, bool)})
 
     Majority filter
 
@@ -518,7 +521,7 @@ def majority_filter(img, N=3, out=None, output=None):
     filtered : ndarray
         boolean image of same size as img.
     '''
-    img = img.astype(np.bool_)
+    img = np.asanyarray(img, dtype=np.bool_)
     output = _get_output(img, out, 'majority_filter', np.bool_, output=output)
     if N <= 1:
         raise ValueError('mahotas.majority_filter: filter size must be positive')
@@ -744,7 +747,7 @@ def tophat_close(f, Bc=None, out=None):
     This returns objects that are smaller than ``Bc`` and contain lower values
     than their surroundings.
 
-    See: http://en.wikipedia.org/wiki/Top-hat_transform
+    See: https://en.wikipedia.org/wiki/Top-hat_transform
 
     Parameters
     ----------
@@ -778,7 +781,7 @@ def tophat_open(f, Bc=None, out=None):
     This returns objects that are smaller than ``Bc`` and contain higher values
     than their surroundings.
 
-    See: http://en.wikipedia.org/wiki/Top-hat_transform
+    See: https://en.wikipedia.org/wiki/Top-hat_transform
 
     Parameters
     ----------
